:orphan:

.. _plugins_in_@{collection_name}@:

Plugin Index
============

These are the plugins in the @{collection_name}@ collection

.. toctree::
    :maxdepth: 1

{% for category, plugins in plugin_maps.items() | sort %}

@{ category | capitalize }@ Plugins
@{ '-' * ((category | length) + 8) }@

{% for name, desc in plugins.items() | sort %}
* :ref:`@{ name }@ <ansible_collections.@{ collection_name }@.@{ name }@_@{ category }@>` -- @{ desc }@
{% endfor %}
{% endfor %}


.. seealso::

    List of :ref:`collections <list_of_collections>` with docs hosted here.
